#! /usr/bin/perl 
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.


use DBI;
use CGI;
use Number::Format qw(:all);

my %lex = ('Teacher' => 'Teacher',
	   'Main' => 'Main',
	   'Timetables' => 'Timetables',
	   'Timetable' => 'Timetable',
	   'Cannot open tex file' => 'Cannot open tex file',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Select' => 'Select',
	   'Lastname, Firstname' => 'Lastname, Firstname',
	   'Homeroom, Lastname, Firstname' => 'Homeroom, Lastname, Firstname',
	   'Grade, Lastname, Firstname' => 'Grade, Lastname, Firstname',
	   'Band, Lastname, Firstname' => 'Band, Lastname, Firstname',
	   'Sort by' => 'Sort by',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'Continue' => 'Continue',
	   'No Staff Found' => 'No Staff Found',
	   'Error' => 'Error',
	   'Blank=All' => 'Blank=All',
	   'Term' => 'Term',
	   'Period' => 'Period',
	   'Day' => 'Day',
	   'Separate with Spaces' => 'Separate with Spaces',
	   'Term(s)' => 'Term(s)',
	   'Font Size' => 'Font Size',
	   'Paper Size' => 'Paper Size',
	   'Letter' => 'Letter',
	   'Legal' => 'Legal',
	   'A4' => 'A4',
	   'Missing' => 'Missing',
	   'Orientation' => 'Orientation',
	   'Landscape' => 'Landscape',
	   'Portrait' => 'Portrait',
	   
	);

my $grayshade = '0.80';
my $maxchars = 10; # maximum characters for subject description (if not small desc field)
my $descfield = 'smdesc'; # either smdesc or description value

my $logowidth = '20mm';
my $logofile = 'schoollogo.jpg';
my $stripampm = 1; # turn off with a zero value.


my $self = "rptstafftimetable.pl";


eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "../../lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $maxlines = 28;
my $shortname = "teachertimetable$$";
my $filename = "$shortname.tex";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;



# print page header
my $title = qq{$lex{Teacher} $lex{Timetables}};
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$css" type="text/css">\n};

print qq{$chartype\n</head><body>\n};
print qq{[ <a href="$homepage">$lex{Main}</a> | \n};
print qq{<a href="$schpage">$lex{Timetable}</a> ]\n};


if ( not $arr{page} ) {
    showStartPage();
    
} else {
    delete $arr{page};
}


#foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

=head
my $includeLocation = $arr{includeLocation};
my $includeSubjsec = $arr{includeSubjsec};
my $includeStudcount = $arr{includeStudcount};

delete $arr{includeLocation};
delete $arr{includeSubjsec};
delete $arr{includeStudcount};
=cut


my $sortorder = "lastname, firstname";

my @terms = split(/\s+/, $arr{term});
delete $arr{term};
print "Terms:@terms<br>\n";


# Now create an array, @staff,  to store userid's of
my (@sort, @skipped, %sort, %staffnames );
my $sth = $dbh->prepare("select lastname, firstname, userid from staff");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

while ( my ($lastname, $firstname,$userid) = $sth->fetchrow ) {
    push @staff, $userid;
    $staffnames{$userid} = "$firstname $lastname";
}

my ( $papersize, $textwidth, $textheight );

if ( @staff ) { # We have staff to print; setup LaTeX file.
    open(TEX,">$filename") || die $lex{'Cannot open tex file'};

    if ( $arr{papersize} eq $lex{Letter} ) {
	$papersize = 'letterpaper';
	$textwidth = $g_letterpaper_textwidth;
	$textheight = $g_letterpaper_textheight;
    } elsif ( $arr{papersize} eq $lex{Legal} ) {
	$papersize = 'legalpaper';
	$textwidth = $g_legalpaper_textwidth;
	$textheight = $g_legalpaper_textheight;
    } elsif ( $arr{papersize} eq $lex{A4} ) {
	$papersize = 'a4paper';
	$textwidth = $g_a4paper_textwidth;
	$textheight = $g_a4paper_textheight;
    } 

    if ( $arr{orientation} eq 'landscape' ) { # flip width and height
	my $temp = $textwidth;
	$textwidth = $textheight;
	$textheight = $temp;
    }

#    print "Text width:$textwidth Text Height:$textheight<br>\n";

    if ( $arr{orientation} eq 'portrait' ) { $arr{orientation} = ''; } # only landscape value
    
    print TEX "\\documentclass[$arr{fontsize}pt,$papersize, $arr{orientation}]{article}
\\usepackage{array,colortbl,multicol,inputenc,graphicx,helvet}
$a_latex_header
\\pagestyle{empty}
\\setlength{\\textwidth}{$textwidth}
\\setlength{\\textheight}{$textheight}
\\setlength{\\evensidemargin}{0mm}
\\setlength{\\oddsidemargin}{0mm}
\\setlength{\\hoffset}{-10mm}
\\setlength{\\voffset}{-25mm}
\\setlength{\\extrarowheight}{2pt}
\\setlength{\\parindent}{0pt}\n";

    print TEX "\\begin{document}\n";

} else {
    print qq{<h3>$lex{'No Staff Found'}/h3>\n};
    print qq{</body></html>\n};
    exit;
}

# Now Do Staff Timetables
my %subtimes; # used for global subject storage;

foreach my $userid ( @staff ) {

    prTimetable( $userid, \@terms ); # do 1 staff member
    
} # End of Staff Loop

print TEX qq{\\end{document}};
close TEX;

system("$pdflatex $filename >pdflog$$.txt");
system("mv $shortname.pdf $downloaddir");
system("mv pdflog$$.txt $downloaddir");
system("rm $shortname.*");

print qq{<h1><a href="$webdownloaddir/$shortname.pdf">\n};
print qq{$lex{'View/Download'} $lex{Teacher} $lex{Timetables}</a></h1>\n};

print qq{[ <a href="$schpage">$lex{Timetable}</a> |\n};
print qq{<a href="$webdownloaddir/pdflog$$.txt">$lex{'View Log File'}\n};
print qq{</a> ]\n};

if ( @skipped ) {
    print qq{<h3>$lex{Missing} $lex{Timetable}</h3>\n};
    print @skipped, qq{<br>\n};
}

print qq{</body></html>\n};


#--------------
sub prTimetable {
#--------------

    my ($userid, $ref) = @_;
    my @terms = @$ref;

    
    # Get staff info
    my $sth = $dbh->prepare("select * from staff where userid = ?");
    $sth->execute( $userid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchrow_hashref;
    my %r = %$ref;

    # Check if they have any timetable info.
    my $skipflag = 1; # skip unless some subjects, timetable found for this teacher

    foreach my $term ( @terms ) {
	my $sth = $dbh->prepare("select count(t.subjsec) from subject s, schedat t where s.teacher = ? and 
         startrptperiod <= '$term' and endrptperiod >= '$term' and s.subjsec = t.subjsec");

	$sth->execute( $userid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $tcount = $sth->fetchrow;
	if ( $tcount ) { # timetable count means we have entries for this term, at least.
	    $skipflag = 0;
	}
    }

    if ( $skipflag ) {  # no timetable (and courses) for these staff members.
	push @skipped, "$r{firstname} $r{lastname}<br>";
	return;
    }
    
    
    # Setup for Get Subject Information
    my $sth2 = $dbh->prepare("select description, smdesc, location, teacher 
      from subject where subjsec = ?");

    my $sth3 = $dbh->prepare("select count(distinct studnum) from eval where subjcode = ?");

    # Now print the Timetable in TeX, heading first.
    print TEX "\\includegraphics[width=$logowidth]{../../etc/$logofile}\\quad\n";

    print TEX "\\parbox[b]{150mm}{ {\\Large $schoolname}\n\n\n\n";
    print TEX "{\\LARGE\\bf $r{firstname} $r{lastname} } ($r{userid})\n\n";
	
    print TEX " }\n\n"; # end of parbox.
    print TEX "\\vspace{8mm}\n";

   
    foreach my $term ( @terms ) {
	
	# Get their courses this term; 
	$sth = $dbh->prepare("select * from subject where teacher = ? and 
          startrptperiod <= '$term' and endrptperiod >= '$term'");
	$sth->execute( $userid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	# Clean up global on later interations.
	foreach my $key ( keys %timetable ) {
	    delete $timetable{$key};
	}

	my %timetable; # timetable{period}{day}{subjsec} = 1
	my $maxdays = 0;

	my $sth1 = $dbh->prepare("select day, period from schedat
           where term = ? and subjsec = ? order by day, period");

	my %periodmap; # stores the mapping values for period times;
	
	while ( my $ref  = $sth->fetchrow_hashref ) { # loop through all courses this term for this teacher.
	    my %s = %$ref;
	    my $subjsec = $s{subjsec};
	    $periodmap{ $g_PeriodMap{$s{grade}} } = 1; # the key has the map value.
	    
	    # load each timetable record for this subject.
	    $sth1->execute( $term, $subjsec ) ;
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my ( $day, $period ) = $sth1->fetchrow ) {
		if ( $day > $maxdays ) { $maxdays = $day; }
		push @{ $timetable{$period}{$day} }, $subjsec;
	    }
	}

	
	# Setup Period Times (into %periodTime{$period} = combined value.
	my @vals = keys %periodmap;
	my $mapvalue = pop @vals;
	my %periodTime;
	my %tempTime = %{ $g_PeriodTime{$mapvalue} };
	foreach my $p ( keys %tempTime ) {
	    my $start = $tempTime{$p}{s};
	    my $end = $tempTime{$p}{e};
	    my ($start,$end) = conv24to12($start,$end);
	    my $val = qq{$start-$end};
	    if ( $stripampm ) { $val =~ s/AM|PM|am|pm//g; }
	    $periodTime{$p} = $val;
	}

	# print qq{<div>MapValue:$mapvalue Periods:}, %periodTimes, "<br>\n";
	
=head	
	# Testing %timetable
	print qq{<h3>Timetable Hash</h3>\n};
	foreach my $period (keys %timetable ) {
	    foreach my $day ( keys %{ $timetable{$period} } ) {
		print qq{Day:$day Period:$period Courses:}, @{ $timetable{$period}{$day} }, qq{<br>\n};
	    }
	}
=cut

	
	if ( not %timetable ) { next; } # term

	my $tempwidth = $textwidth;
	$tempwidth =~ s/mm//g; # strip mm text.
	my $cellwidth = ( $tempwidth - 25 ) / $maxdays;
	$cellwidth = round($cellwidth - 5, 1);
#	print "Temp:$tempwidth DAYS:$maxdays CELL WIDTH = $cellwidth<br>\n";
	if ( $cellwidth > 60 ) { $cellwidth = 60; }

	
	# Tabular Start
	print TEX "\\begin{tabular}{|p{23mm}|";
	for ( 1 .. $maxdays ) { print TEX "p{$cellwidth mm}|" }
	print TEX "}\\hline\n\n";

	
	# Print Top Row here.
	print TEX "\\rowcolor[gray]{0.92}$lex{Term} $term / ". $lex{Day};

	my $dayvalue;
	for my $day ( 1 .. $maxdays ) { 
	    if ( $g_DaysPerCycle == 5 or $g_DaysPerCycle eq 'W' or $g_DaysPerCycle eq 'w' ) {  # weekly
		$dayvalue = $dow[$day + 1];
	    } else {
		$dayvalue = $day;
	    }
	    print TEX " & \\hfil $dayvalue \\hfil"; 
	}
	print TEX "\\\\ \\hline\n";

	my @periods = sort { $a <=> $b } keys %timetable;
	my $maxperiods = $periods[-1];

#	print "Max Days:$maxdays Max Periods:$maxperiods<br>\n";


	foreach my $period ( 1 .. $maxperiods ) {
	    

	    print TEX qq{$lex{Period} $period\n\n $periodTime{$period} &};
	    
	    foreach my $day ( 1 .. $maxdays ) {

		my $cellvalue;
		while ( my $subjsec = pop @{ $timetable{$period}{$day} } ) {

#		    print "Period:$period Day:$day SUB:$subjsec<br>\n";
		    
		    # Load subject information.
		    $sth2->execute( $subjsec );
		    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		    my $ref = $sth2->fetchrow_hashref;
		    # Filter for strange characters.
		    foreach my $key ( keys %$ref ) {
			($ref->{$key}) = latex_filter($ref->{$key});
		    }

		    %r = %$ref;
		    if ( not $r{smdesc} ) {
			$r{smdesc} = substr( $r{description}, $maxchars ); # defined at top.
		    }
			    
		    # Load course enrollment
		    $sth3->execute( $subjsec );
		    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		    $studcount = $sth3->fetchrow;

		    # $arr{desctype} is either 'description' or 'smdesc' field
		    my $coursedesc = $r{"$arr{desctype}"};
		    if ( length $coursedesc > $arr{descsize} ) { # truncate.
			$coursedesc = substr( $coursedesc, 0, $arr{descsize} );
		    }

		    $cellvalue .= "{\\bf $coursedesc}\n\n";
		    if ( $arr{includeSubjsec} ) { $cellvalue .= qq{$subjsec }; }
		    if ( $arr{includeStudcount} and $studcount ) { $cellvalue .= qq{($studcount)}; }
		    if ( $arr{includeLocation} and $r{location} ) { 
			$cellvalue .= qq{\n\n$r{location}}; 
		    } else { 
			$cellvalue .= qq{\n\n}
		    }; 

		}

		$cellvalue =~ s/\n+$//g; # strip any trailing newlines.
		print TEX qq{$cellvalue};

		if ( $day != $maxdays ) {
		    print TEX qq{&};
		}

	    } # end of day loop.
	    print TEX qq{\\\\\\hline\n\n};	    

	} # end of period loop
	
	print TEX qq{\\end{tabular}\n\n};
	
	
    } # end of Term Loop
    
    print TEX qq{\n\\newpage\n}; # new page for next teacher.

} # end of prTimetable




#-----------------
sub showStartPage {
#-----------------


    # Get default papersize
    my $papersize;
    if ( $defaultpapersize ) {
	$papersize = $defaultpapersize;
	$papersize =~ s/paper//; # strip off the 'paper' ending;
	$papersize = ucfirst( $papersize );
    }

    # print sortorder and selection input form.
    print qq{<h1>$title</h1>\n};
    print qq{<div><form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};

    print qq{<table cellpadding="3" cellspacing="0" border="0">\n};


    # Terms
    print qq{<tr><td class="bra">$lex{'Term(s)'}</td><td>};
    print qq{<input type="text" name="term" size="12">\n};
    print qq{$lex{'Separate with Spaces'}</td></tr>\n};

    
    # Paper Size
    print qq{<tr><td class="bra">$lex{'Paper Size'}</td>\n};
    print qq{<td align="left"><select name="papersize">};
    if ( $papersize ) { print qq{<option>$papersize</option>\n}; }
    print qq{<option>$lex{Letter}</option>};
    print qq{<option>$lex{A4}</option>};
    print qq{<option>$lex{Legal}</option>};
    print qq{</select></td></tr>\n};

    # Font Size
    print qq{<tr><td class="bra">$lex{'Font Size'}</td>\n};
    print qq{<td><select name="fontsize"><option>10</option><option>11</option>\n};
    print qq{<option>12</option></select></td></tr>\n};

    # Orientation
    print qq{<tr><td class="bra">$lex{Orientation}</td>\n};
    print qq{<td><select name="orientation"><option value="landscape">$lex{Landscape}</option>};
    print qq{<option value="portrait">$lex{Portrait}</option>};
    print qq{<option></option></select></td></tr>\n};
    
    # Description cell
    print qq{<tr><td class="bra">Course Description Size</td>};
    print qq{<td><select name="desctype"><option value="description">Full</option>};
    print qq{<option value="smdesc">Short</option>};
    print qq{</select></td></tr>\n};

    # Max Characters.
    print qq{<tr><td class="bra">Max Description Characters</td>};
    print qq{<td><input type="text" name="descsize" style="width:2em;" value="25">\n};
    print qq{ (for Full Description)</td></tr>\n};

    
    # Things to Include: location, count, subjsec
    my $checked = qq{checked="checked"};
   
    print qq{<tr><td class="bra">Include Course Code-Section</td>};
    print qq{<td class="la"><input type="checkbox" name="includeSubjsec" value="1" $checked> };
    print qq{</td></tr>\n};

    print qq{<tr><td class="bra">Include Student Count</td>};
    print qq{<td class="la"><input type="checkbox" name="includeStudcount" value="1"></td></tr>\n}; 
    
    print qq{<tr><td class="bra">Include Location</td>};
    print qq{<td class="bla"><input type="checkbox" name="includeLocation" value="1"> };
    print qq{</td></tr>\n};

    print qq{<tr><td class="cn" colspan="2">};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};

    print qq{</table></form></div>\n};
    print qq{</body></html>\n};

    exit;
}


#-------------
sub conv24to12 {
#-------------

    my @times;
    while ( my $time = shift ) {

#	if ( not $time ) { return }
	$time =~ s/\s+//g; # strip any spaces.
	my ($hr, $min) = split(':', $time);
	my $ampm = 'AM';
	if ( $hr == 12 ) { $ampm = 'PM'; }
	if ( $hr == 0 ) { $hr = 12; };

	if ( $hr > 12 ) { 
	    $hr -= 12;
	    $ampm = 'PM';
	}

	my $t = qq{$hr:$min$ampm};
	push @times, $t;
    }

    return @times;

}
